import React, { createElement, FC, useState } from 'react'
import { Route, RouteProps } from 'react-router'
import {
  access,
  authenticate,
  blockVisitor,
  forwardToLogin
} from './index'

/**
 * 定义一个需要用户登录才能使用的路由
 * 当前自定义的路由组件是业务公用组件
 * https://reacttraining.com/react-router/web/example/auth-workflow
 */
const AuthRoute: FC<RouteProps> = (routeProps) => {
  const {
    children, component, render,
    ...rest
  } = routeProps
  const [RouteComponent, updateComponent] = useState(null)
  // 兼容Route组件接口，拿到最终组件实例
  const getComponent = (props) => {
    let _component
    /**
     * children, component, render按照React Router文档描述按优先级排列
     * https://reacttraining.com/react-router/web/api/Route/route-props
     */
    if (children) {
      // children已经被转义，可直接作为返回值
      _component = children
    } else if (component) {
      // component没有被转义，需要以jsx(React.createElement)作为返回值
      _component = createElement(component, props) // <component {...props} />
    } else if (render) {
      // render是函数，需要调用后作为返回值
      _component = render(props)
    }

    return _component
  }
  const renderFn = props => {
    if (RouteComponent) {
      return RouteComponent
    }

    authenticate()
      .then(() => access(routeProps) // 已经登录，进入访问控制
        .then(() => updateComponent(getComponent(props))) // 允许通过
        .catch(() => blockVisitor()))
      .catch(() => forwardToLogin()) // 如果未登录

    // react组件必须有个返回
    return null
  }

  return <Route {...rest} render={renderFn} />
}

export default AuthRoute
